/*******************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.data.ui.dataset;

import org.eclipse.birt.report.designer.data.ui.property.AbstractDescriptionPropertyPage;
import org.eclipse.birt.report.designer.nls.Messages;
import org.eclipse.birt.report.model.api.activity.SemanticException;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

/**
 * 
 */

public class DataSetSettingsPage extends AbstractDescriptionPropertyPage
{

	private static String DEFAULT_MESSAGE = Messages.getString( "dataset.editor.settings" );
	private transient Button fetchAllDataCheckBox = null;
	
	public Control createContents( Composite parent )
	{
		Composite composite = new Composite( parent, SWT.NONE );

		GridLayout layout = new GridLayout( );
		layout.numColumns = 1;
		composite.setLayout( layout );
		composite.setLayoutData( new GridData( GridData.HORIZONTAL_ALIGN_FILL
				| GridData.VERTICAL_ALIGN_FILL ) );

		addDataFetchSettingGroup( composite );
		return composite;
	}

	/**
	 * Add row fetch limit control group.
	 * 
	 * @param composite
	 */
	private void addDataFetchSettingGroup( Composite composite )
	{
		GridLayout groupGridLayout = new GridLayout( );
		groupGridLayout.numColumns = 5;
		GridData groupGridData = new GridData( GridData.FILL_HORIZONTAL );

		Group dataFetchSettingGroup = new Group( composite, SWT.NONE );
		dataFetchSettingGroup.setLayoutData( groupGridData );
		dataFetchSettingGroup.setLayout( groupGridLayout );
		dataFetchSettingGroup.setText( Messages.getString( "dataset.editor.settings.dataFetchSetting" ) );

		fetchAllDataCheckBox = new Button( dataFetchSettingGroup, SWT.CHECK );
		GridData data = new GridData( );
		data.horizontalSpan = 5;
		fetchAllDataCheckBox.setLayoutData( data );
		fetchAllDataCheckBox.setText( Messages.getString( "dataset.editor.settings.dataFetchSetting.fetchAll" ) );

		final Label dataFetchLabel = new Label( dataFetchSettingGroup, SWT.NONE );
		dataFetchLabel.setText( Messages.getString( "SettingsPage.dataFetchSetting.label" ) );

		final Text rowFetchLimitText = new Text( dataFetchSettingGroup,
				SWT.BORDER );
		GridData gData = new GridData( );
		gData.horizontalSpan = 2;
		gData.widthHint = 50;
		rowFetchLimitText.setLayoutData( gData );
		
		if ( getDataSetRowFetchLimit( ) > 0 )
		{
			fetchAllDataCheckBox.setSelection( false );
			rowFetchLimitText.setEnabled( true );
			dataFetchLabel.setEnabled( true );
			rowFetchLimitText.setText( Integer.toString( getDataSetRowFetchLimit( ) ) );
		}
		else
		{
			fetchAllDataCheckBox.setSelection( true );
			rowFetchLimitText.setEnabled( false );
			dataFetchLabel.setEnabled( false );
			rowFetchLimitText.setText( "" );

		}

		fetchAllDataCheckBox.addSelectionListener( new SelectionAdapter( ) {

			public void widgetSelected( SelectionEvent e )
			{

				final boolean isSelection = fetchAllDataCheckBox.getSelection( );
				dataFetchLabel.setEnabled( !isSelection );
				rowFetchLimitText.setEnabled( !isSelection );

				if ( isSelection )
				{
					rowFetchLimitText.setText( "" ); 
				}

			}
		} );

		rowFetchLimitText.addModifyListener( new ModifyListener( ) {

			public void modifyText( ModifyEvent e )
			{
				int rowFetchLimit = 0;

				try
				{

					if ( isNumber( rowFetchLimitText.getText( ) ) )
					{
						String rowLimitText = rowFetchLimitText.getText( );
						if (rowLimitText.trim( )=="")
							rowLimitText="0";
						rowFetchLimit = new Double( rowLimitText ).intValue( );
						rowFetchLimit = rowFetchLimit < 0 ? 0 : rowFetchLimit;

						setDataSetRowFetchLimit( rowFetchLimit );

						getContainer( ).setMessage( DEFAULT_MESSAGE,
								IMessageProvider.NONE );
					}
					else
					{
						getContainer( ).setMessage( Messages.getString( "dataset.editor.settings.dataFetchSetting.errorNumberFormat" ),
								IMessageProvider.ERROR );
					}

				}
				catch ( SemanticException e1 )
				{
					getContainer( ).setMessage( Messages.getString( "dataset.editor.settings.dataFetchSetting.errorNumberFormat" ),
							IMessageProvider.ERROR );
				}

			}

		} );

	}

	/**
	 * Test the text to see if it can be parsed to an integer.
	 * 
	 * @param text
	 * @return
	 */
	private boolean isNumber( String text )
	{
		if (text.trim( )=="")
			return true;
		if ( text == null || text.trim( ).length( ) == 0 )
			return false;
		return text.matches( "^[0-9]*[1-9][0-9]*$" );
		
	}

	/**
	 * 
	 * @param count
	 * @throws SemanticException
	 */
	private void setDataSetRowFetchLimit( int count ) throws SemanticException
	{
		( (DataSetEditor) getContainer( ) ).getHandle( )
				.setRowFetchLimit( count );
	}

	/**
	 * 
	 * @return
	 */
	private int getDataSetRowFetchLimit( )
	{
		return ( (DataSetEditor) getContainer( ) ).getHandle( )
				.getRowFetchLimit( );

	}

	public void pageActivated( )
	{
		getContainer( ).setMessage( DEFAULT_MESSAGE, IMessageProvider.NONE );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractPropertyPage#performCancel()
	 */
	public boolean performCancel( )
	{
		// selectorImage.dispose( );
		return super.performCancel( );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractPropertyPage#canLeave()
	 */
	public boolean canLeave( )
	{
		if ( canFinish( ) )
			return super.performOk( );
		else
			return false;
	}

	/**
	 * whether the page can be finished or leave.
	 */
	public boolean canFinish( )
	{
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.ui.dialogs.properties.AbstractPropertyPage#performOk()
	 */
	public boolean performOk( )
	{
		if ( canFinish( ) )
		{
			return super.performOk( );
		}
		else
			return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.data.ui.property.AbstractPropertyPage#getToolTip()
	 */
	public String getToolTip( )
	{
		return Messages.getString( "SettingsPage.CachePreference.Filter.Tooltip" ); //$NON-NLS-1$
	}

}
