/*************************************************************************************
 * Copyright (c) 2004 Actuate Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Actuate Corporation - Initial implementation.
 ************************************************************************************/

package org.eclipse.birt.report.designer.internal.ui.dialogs.js;

import org.eclipse.birt.report.designer.internal.ui.editors.DocumentProvider;
import org.eclipse.birt.report.designer.internal.ui.editors.StorageDocumentProvider;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.rules.DefaultPartitioner;

/**
 * Subclass of <code>DocumentProvider</code>, creates documents and sets
 * partition scanners
 * 
 * 
 */
public class JSDocumentProvider extends StorageDocumentProvider
{

	/**
	 * Array of token types
	 */
	private static String[] colorTokens = {
			JSPartitionScanner.JS_COMMENT,
			JSPartitionScanner.JS_STRING,
			JSPartitionScanner.JS_KEYWORD
	};

	public JSDocumentProvider( )
	{
		super( );
	}

	/**
	 * @see DocumentProvider#createDocument(java.lang.Object)
	 */
	protected IDocument createDocument( Object element ) throws CoreException
	{
		IDocument document = super.createDocument( element );
		if ( document != null )
		{
			IDocumentPartitioner partitioner = new DefaultPartitioner( new JSPartitionScanner( ),
					colorTokens );
			partitioner.connect( document );
			document.setDocumentPartitioner( partitioner );
		}
		return document;
	}

}